class APIKeyAuth:
    def __init__(
        self,
        api_key,
    ) -> None:
        self.api_key = api_key


class UnamePassAuth:
    def __init__(
        self,
        username,
        password,
    ) -> None:
        self.username = username
        self.password = password



class Response:
    """an unified Response model for every panels"""

    __dict__ = ("status_code", "real_resp")

    def __init__(
        self,
        status_code: int,
        real_resp: dict,
    ) -> None:
        """TODO write doc string"""
        self.status_code = status_code
        self.real_resp = real_resp

    def __eq__(self, other) -> bool:
        """checks the equality of two Response object"""
        if isinstance(other, Response):
            return self.status_code == other.status_code
        raise TypeError

    def get_real_resp(self):
        return self.real_resp



import json

import requests

from prsmsp.abstracts.abcpanel import ABCSmsPanel
from prsmsp.factories import AuthFactory
from prsmsp.models import Response


class SmsOne(ABCSmsPanel):
    def __init__(self, api_key: str) -> None:
        """Take the auth info

        :param api_key: your smsone token auth
        :type api_key: str

        :rtype None
        :return: None
        """
        self.auth = AuthFactory.get("api_key")(api_key)

    def _response_parser(self, resp: requests.Response) -> Response:
        status_code = int(resp.status_code)
        real_response = json.loads(resp.text)

        return Response(status_code, real_response)

    def send_sms(
        self, receptor: str, message: str
    ) -> Response:  # NoQA
        """send sms with smsone sms panel

        :param receptor: reciver of your message
        :type receptor: str

        :param message: the message you want to send
        :type message: str

        :param originator: the originator that you want to send your message
        :type originator: str

        :rtype Response
        :return: The requests response
        """

        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.auth.api_key}"
        }

        url = "https://app.sms1.ir:7001/api/service/send"

        data = {
            "message": message,
            "recipient": receptor,
        }

        resp = requests.post(url, headers=headers, json=data)

        return self._response_parser(resp)


import json

import requests

from prsmsp.abstracts.abcpanel import ABCSmsPanel
from prsmsp.factories import AuthFactory
from prsmsp.models import Response


class NikSms(ABCSmsPanel):
    def __init__(self, username, password) -> None:
        """Take the auth info

        :param username: webonesms username
        :type username: str

        :param password: webonesms password
        :type password: str

        :rtype None
        :return: None
        """
        self.auth = AuthFactory.get("username_pass")(username, password)

    def _response_parser(self, resp: requests.Response) -> Response:
        status_code = int(resp.status_code)
        real_response = json.loads(resp.text)

        return Response(status_code, real_response)

    def send_sms(self, receptor: str, message: str, originator: str = "") -> Response:
        """send sms with niksms sms panel

        :param receptor: reciver of your message
        :type receptor: str

        :param message: the message you want to send
        :type message: str

        :param originator: the originator that you want to send your message (optional)
        :type originator: str

        :rtype Response
        :return: The requests response
        """

        url = "https://niksms.com/fa/PublicApi/PtpSms"

        data = {
            "numbers": [receptor],
            "message": message,
            "username": self.auth.username,
            "password": self.auth.password,
        }

        if originator :
            data["senderNumber"] = originator

        resp = requests.post(url, json=data)

        return self._response_parser(resp)



from prsmsp.panels import *

class PanelFactory:

    @staticmethod
    def get(panel_name: str):

        PANELS = {
            "kavenegar": Kavenegar,
            "smsdotir": SmsDotIr,
            "webonesms": WebOneSms,
            "melipayamak": MeliPayamak,
            "mediana": Mediana,
            "ghasedaksms": GhasedakSms,
            "farazsms": FarazSms,
            "niksms": NikSms,
            "smsone": SmsOne,
            "sapak": Sapak,
        }

        p = PANELS.get(panel_name, None)
        print(p)
        if p:
            return p

        raise KeyError("Panel Not Found")


class Panel:

    staticmethod
    def initiate(panel_name: str, **auth):
        p = PanelFactory.get(panel_name)

        return p(*auth.values())


from .kavenegar import Kavenegar
from .melipayamak import MeliPayamak
from .smsdotir import SmsDotIr
from .webonesms import WebOneSms
from .mediana import Mediana
from .ghasedaksms import GhasedakSms
from .farazsms import FarazSms
from .niksms import NikSms
from .smsone import SmsOne
from .sapak import Sapak

__all__ = [
    "Kavenegar",
    "SmsDotIr",
    "WebOneSms",
    "MeliPayamak",
    "Mediana",
    "GhasedakSms",
    "FarazSms",
    "NikSms",
    "SmsOne",
    "Sapak",
]
